@@ -31,13 +31,25 @@ class RedisBackend(Timeseries):
     return Timeseries.__new__(cls, *args, **kwargs)
 
   def __init__(self, client, **kwargs):
-    super(RedisBackend,self).__init__( client, **kwargs )
-
     # prefix is redis-only feature (TODO: yes or no?)
     self._prefix = kwargs.get('prefix', '')
     if len(self._prefix) and not self._prefix.endswith(':'):
       self._prefix += ':'
 
+    super(RedisBackend,self).__init__( client, **kwargs )
+
+  def _calc_keys(self, config, name, timestamp):
+    '''
+    Calculate keys given a stat name and timestamp.
+    '''
+    i_bucket = config['i_calc'].to_bucket( timestamp )
+    r_bucket = config['r_calc'].to_bucket( timestamp )
+
+    i_key = '%s%s:%s:%s'%(self._prefix, name, config['interval'], i_bucket)
+    r_key = '%s:%s'%(i_key, r_bucket)
+
+    return i_bucket, r_bucket, i_key, r_key
+
   def _insert(self, name, value, timestamp):
     '''
     Insert the value.
@@ -46,7 +58,7 @@ class RedisBackend(Timeseries):
     # TODO: apply the prefix if we're using one.
 
     for interval,config in self._intervals.iteritems():
-      i_bucket, r_bucket, i_key, r_key = config['calc_keys'](name, timestamp)
+      i_bucket, r_bucket, i_key, r_key = self._calc_keys(config, name, timestamp)
       
       if config['coarse']:
         self._type_insert(pipe, i_key, value)
@@ -85,12 +97,12 @@ class RedisBackend(Timeseries):
     '''
     Fetch a single interval from redis.
     '''
-    i_bucket, r_bucket, i_key, r_key = config['calc_keys'](name, timestamp)
+    i_bucket, r_bucket, i_key, r_key = self._calc_keys(config, name, timestamp)
     
     rval = OrderedDict()    
     if config['coarse']:
       data = self._process_row( self._type_get(self._client, i_key) )
-      rval[ i_bucket*config['step'] ] = data
+      rval[ config['i_calc'].from_bucket(i_bucket) ] = data
     else:
       # First fetch all of the resolution buckets for this set.
       resolution_buckets = sorted(map(int,self._client.smembers(i_key)))
@@ -105,7 +117,7 @@ class RedisBackend(Timeseries):
 
       for idx,data in enumerate(res):
         data = self._process_row(data)
-        rval[ resolution_buckets[idx]*config['resolution'] ] = data
+        rval[ config['r_calc'].from_bucket(resolution_buckets[idx]) ] = data
 
     return rval
 
@@ -135,9 +147,9 @@ class RedisBackend(Timeseries):
 
       if config['coarse']:
         data = self._process_row( data )
-        rval[interval_bucket*step] = data
+        rval[ config['i_calc'].from_bucket(interval_bucket) ] = data
       else:
-        rval[interval_bucket*step] = OrderedDict()
+        rval[ config['i_calc'].from_bucket(interval_bucket) ] = OrderedDict()
         pipe = self._client.pipeline(transaction=False)
         resolution_buckets = sorted(map(int,data))
         for bucket in resolution_buckets:
@@ -147,8 +159,9 @@ class RedisBackend(Timeseries):
         
         resolution_res = pipe.execute()
         for x,data in enumerate(resolution_res):
-          rval[interval_bucket*step][ resolution_buckets[x]*resolution ] = \
-            self._process_row(data)
+          i_t = config['i_calc'].from_bucket(interval_bucket)
+          r_t = config['r_calc'].from_bucket(resolution_buckets[x])
+          rval[ i_t ][ r_t ] = self._process_row(data)
 
     return rval
 
