@@ -371,11 +371,19 @@ class Timeseries(object):
     if not config:
       raise UnknownInterval(interval)
 
-    rval = self._get( name, interval, config, timestamp )
+    # If name is a list, then join all of results. It is more efficient to
+    # use a single data structure and join "in-line" but that requires a major
+    # refactor of the backends, so trying this solution to start with. At a
+    # minimum we'd have to rebuild the results anyway because of the potential
+    # for sparse data points would result in an out-of-order result.
+    if isinstance(name, (list,tuple,set)):
+      results = [ self._get(x, interval, config, timestamp) for x in name ]
+      # Even resolution data is "coarse" in that it's not nested
+      rval = self._join_results( results, True )
+    else:
+      rval = self._get( name, interval, config, timestamp )
 
     # If condensed, collapse the result into a single row
-    # TODO: figure out how to not recalculate this value that subclasses will
-    # also be calculating
     if condensed and not config['coarse']:
       rval = { config['i_calc'].normalize(timestamp) : self._condense(rval) }
     if transform:
@@ -389,7 +397,7 @@ class Timeseries(object):
     '''
     raise NotImplementedError()
   
-  def series(self, name, interval, steps=None, condensed=False, start=None, end=None, transform=None):
+  def series(self, name, interval, steps=None, condensed=False, start=None, end=None, transform=None, collapse=False):
     '''
     Return all the data in a named time series for a given interval. If steps
     not defined and there are none in the config, defaults to 1.
@@ -409,6 +417,9 @@ class Timeseries(object):
       raise UnknownInterval(interval)
     steps = steps if steps else config.get('steps',1)
 
+    # If collapse, also condense
+    if collapse: condensed=True
+
     # Fugly range determination, all to get ourselves a start and end 
     # timestamp. Adjust steps argument to include the anchoring date.
     if not end:
@@ -432,25 +443,73 @@ class Timeseries(object):
     end = config['i_calc'].from_bucket( end_bucket )
     
     interval_buckets = config['i_calc'].buckets(start, end)
-    rval = self._series(name, interval, config, interval_buckets)
 
-    if config['coarse'] and transform:
-      for key,data in rval.iteritems():
-        rval[key] = self._process_transform(data, transform)
+    # If name is a list, then join all of results. It is more efficient to
+    # use a single data structure and join "in-line" but that requires a major
+    # refactor of the backends, so trying this solution to start with. At a
+    # minimum we'd have to rebuild the results anyway because of the potential
+    # for sparse data points would result in an out-of-order result.
+    if isinstance(name, (list,tuple,set)):
+      results = [ self._series(x, interval, config, interval_buckets) for x in name ]
+      rval = self._join_results( results, config['coarse'] )
+    else:
+      rval = self._series(name, interval, config, interval_buckets)
+
+    # If fine-grained, first do the condensed pass so that it's easier to do
+    # the collapse afterwards. Be careful not to run the transform if there's
+    # going to be another pass at condensing the data.
     if not config['coarse']:
       if condensed:
         for key in rval.iterkeys():
           data = self._condense( rval[key] )
-          if transform:
+          if transform and not collapse:
             data = self._process_transform(data, transform)
           rval[key] = data
       elif transform:
         for interval,resolutions in rval.iteritems():
           for key in resolutions.iterkeys():
             resolutions[key] = self._process_transform(resolutions[key], transform)
+
+    if config['coarse'] or collapse:
+      if collapse:
+        data = self._condense(rval)
+        if transform:
+          rval = { rval.keys()[0] : self._process_transform(data, transform) }
+        else:
+          rval = { rval.keys()[0] : data }
+
+      elif transform:
+        for key,data in rval.iteritems():
+          rval[key] = self._process_transform(data, transform)
     
     return rval
 
+  def _series(self, name, interval, config, buckets):
+    '''
+    Subclasses must implement fetching a series.
+    '''
+    raise NotImplementedError()
+
+  def _join_results(self, results, coarse):
+    '''
+    Join a list of results. Supports both get and series.
+    '''
+    rval = OrderedDict()
+    i_keys = set()
+    for res in results:
+      i_keys.update( res.keys() )
+    for i_key in sorted(i_keys):
+      if coarse:
+        rval[i_key] = self._join( [res.get(i_key) for res in results] )
+      else:
+        rval[i_key] = OrderedDict()
+        r_keys = set()
+        for res in results:
+          r_keys.update( res.get(i_key,{}).keys() )
+        for r_key in sorted(r_keys):
+          rval[i_key][r_key] = self._join( [res.get(i_key,{}).get(r_key) for res in results] )
+    return rval
+
   def _process_transform(self, data, transform):
     '''
     Process transforms on the data.
@@ -473,12 +532,6 @@ class Timeseries(object):
     Subclasses must implement inserting a value for a key.
     '''
     raise NotImplementedError()
-    
-  def _series(self, name, interval, config, buckets):
-    '''
-    Subclasses must implement fetching a series.
-    '''
-    raise NotImplementedError()
 
   def _process_row(self, data):
     '''
@@ -495,6 +548,12 @@ class Timeseries(object):
     '''
     raise NotImplementedError()
 
+  def _join(self, rows):
+    '''
+    Join multiple rows worth of data into a single result.
+    '''
+    raise NotImplementedError()
+
 
 class Series(Timeseries):
   '''
@@ -535,6 +594,15 @@ class Series(Timeseries):
       return reduce(operator.add, data.values())
     return []
 
+  def _join(self, rows):
+    '''
+    Join multiple rows worth of data into a single result.
+    '''
+    rval = []
+    for row in rows:
+      if row: rval.extend( row )
+    return rval
+
 class Histogram(Timeseries):
   '''
   Data for each interval is stored in a hash, counting occurrances of the
@@ -580,6 +648,17 @@ class Histogram(Timeseries):
         rval[ value ] = count + rval.get(value,0)
     return rval
 
+  def _join(self, rows):
+    '''
+    Join multiple rows worth of data into a single result.
+    '''
+    rval = {}
+    for row in rows:
+      if row:
+        for value,count in row.iteritems():
+          rval[ value ] = count + rval.get(value,0)
+    return rval
+
 class Count(Timeseries):
   '''
   Time series that simply increments within each interval.
@@ -608,6 +687,15 @@ class Count(Timeseries):
       return sum(data.values())
     return 0
 
+  def _join(self, rows):
+    '''
+    Join multiple rows worth of data into a single result.
+    '''
+    rval = 0
+    for row in rows:
+      if row: rval += row
+    return rval
+
 class Gauge(Timeseries):
   '''
   Time series that stores the last value.
@@ -629,11 +717,22 @@ class Gauge(Timeseries):
 
   def _condense(self, data):
     '''
-    Condense by adding together all of the lists.
+    Condense by returning the last real value of the gauge.
     '''
     if data:
-      return data.values()
-    return []
+      data = filter(None,data.values())
+      if data:
+        return data[-1]
+    return None
+
+  def _join(self, rows):
+    '''
+    Join multiple rows worth of data into a single result.
+    '''
+    rval = None
+    for row in rows:
+      if row: rval = row
+    return rval
 
 # Load the backends after all the timeseries had been defined.
 try:
